"""Starknet blocks, txs, labels

Revision ID: 6f8c4b9e3970
Revises: 0f8ee1ebb45f
Create Date: 2023-10-31 13:46:35.536476

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '6f8c4b9e3970'
down_revision = '0f8ee1ebb45f'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('starknet_blocks',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('sequencer_address', sa.VARCHAR(length=256), nullable=True),
    sa.Column('status', sa.VARCHAR(length=256), nullable=True),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=True),
    sa.Column('new_root', sa.VARCHAR(length=256), nullable=True),
    sa.Column('timestamp', sa.BigInteger(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_starknet_blocks'))
    )
    op.create_index(op.f('ix_starknet_blocks_block_number'), 'starknet_blocks', ['block_number'], unique=True)
    op.create_index(op.f('ix_starknet_blocks_hash'), 'starknet_blocks', ['hash'], unique=False)
    op.create_index(op.f('ix_starknet_blocks_timestamp'), 'starknet_blocks', ['timestamp'], unique=False)
    op.create_table('starknet_labels',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('label', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=True),
    sa.Column('address', sa.VARCHAR(length=256), nullable=True),
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=True),
    sa.Column('label_data', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=True),
    sa.Column('created_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_starknet_labels')),
    sa.UniqueConstraint('id', name=op.f('uq_starknet_labels_id'))
    )
    op.create_index(op.f('ix_starknet_labels_address'), 'starknet_labels', ['address'], unique=False)
    op.create_index(op.f('ix_starknet_labels_block_number'), 'starknet_labels', ['block_number'], unique=False)
    op.create_index(op.f('ix_starknet_labels_block_timestamp'), 'starknet_labels', ['block_timestamp'], unique=False)
    op.create_index(op.f('ix_starknet_labels_label'), 'starknet_labels', ['label'], unique=False)
    op.create_index(op.f('ix_starknet_labels_transaction_hash'), 'starknet_labels', ['transaction_hash'], unique=False)
    op.create_table('starknet_transactions',
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('calldata', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.Column('address', sa.VARCHAR(length=256), nullable=False),
    sa.Column('entry_point_selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('max_fee', sa.Numeric(precision=78, scale=0), nullable=True),
    sa.Column('nonce', sa.VARCHAR(length=256), nullable=True),
    sa.Column('signature', sa.ARRAY(sa.Text()), nullable=True),
    sa.Column('type', sa.VARCHAR(length=256), nullable=True),
    sa.Column('version', sa.VARCHAR(length=256), nullable=True),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_starknet_transactions'))
    )
    op.create_index(op.f('ix_starknet_transactions_address'), 'starknet_transactions', ['address'], unique=False)
    op.create_index(op.f('ix_starknet_transactions_hash'), 'starknet_transactions', ['hash'], unique=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_starknet_transactions_hash'), table_name='starknet_transactions')
    op.drop_index(op.f('ix_starknet_transactions_address'), table_name='starknet_transactions')
    op.drop_table('starknet_transactions')
    op.drop_index(op.f('ix_starknet_labels_transaction_hash'), table_name='starknet_labels')
    op.drop_index(op.f('ix_starknet_labels_label'), table_name='starknet_labels')
    op.drop_index(op.f('ix_starknet_labels_block_timestamp'), table_name='starknet_labels')
    op.drop_index(op.f('ix_starknet_labels_block_number'), table_name='starknet_labels')
    op.drop_index(op.f('ix_starknet_labels_address'), table_name='starknet_labels')
    op.drop_table('starknet_labels')
    op.drop_index(op.f('ix_starknet_blocks_timestamp'), table_name='starknet_blocks')
    op.drop_index(op.f('ix_starknet_blocks_hash'), table_name='starknet_blocks')
    op.drop_index(op.f('ix_starknet_blocks_block_number'), table_name='starknet_blocks')
    op.drop_table('starknet_blocks')
    # ### end Alembic commands ###
